@@ -30,6 +30,15 @@ class RecipeItem:
     def __lt__(self, other: "RecipeItem") -> bool:
         return self.value < other.value
 
+    def __le__(self, other: "RecipeItem") -> bool:
+        return self.value <= other.value
+
+    def __gt__(self, other: "RecipeItem") -> bool:
+        return self.value > other.value
+
+    def __ge__(self, other: "RecipeItem") -> bool:
+        return self.value >= other.value
+
     def __eq__(self, other: Union[str, int, "RecipeItem"]) -> bool:
         if isinstance(other, RecipeItem):
             return self.value == other.value and self.selector == other.selector
